@@ -12,7 +12,7 @@ use Digest::SHA qw(hmac_sha256 hmac_sha256_hex);
 use MIME::Base64::URLSafe qw(urlsafe_b64decode);
 use Scalar::Util qw(blessed);
 
-our $VERSION = '1.13';
+our $VERSION = '1.20';
 
 sub new {
     my $class = shift;
@@ -176,8 +176,6 @@ sub set_access_token {
 sub get_app_token {
     my $self = shift;
 
-    croak 'app_id and secret must be set' unless $self->app_id && $self->secret;
-
     # Document does not mention what grant_type is all about or what values can
     # be set, but RFC 6749 covers the basic idea of grant types and Section 4.4
     # describes Client Credentials Grant.
@@ -186,7 +184,7 @@ sub get_app_token {
 }
 
 # Manually Build a Login Flow > Confirming identity > Exchanging code for an access token
-# https://developers.facebook.com/docs/facebook-login/login-flow-for-web-no-jssdk/#exchangecode
+# https://developers.facebook.com/docs/facebook-login/manually-build-a-login-flow
 sub get_user_token_by_code {
     my ($self, $code) = @_;
 
@@ -203,6 +201,36 @@ sub get_user_token_by_code {
     return $token_ref;
 }
 
+sub get_user_token_by_cookie {
+    my ($self, $cookie_value) = @_;
+
+    croak 'cookie value is not given' unless $cookie_value;
+
+    my $parsed_signed_request = $self->parse_signed_request($cookie_value);
+    
+    # https://github.com/oklahomer/p5-Facebook-OpenGraph/issues/1#issuecomment-41065480
+    # parsed content should be something like below.
+    # {
+    #     algorithm => "HMAC-SHA256",
+    #     issued_at => 1398180151,
+    #     code      => "SOME_OPAQUE_STRING",
+    #     user_id   => 44007581,
+    # };
+    croak q{"code" is not contained in cookie value: } . $cookie_value
+        unless $parsed_signed_request->{code};
+
+    # Redirect_uri MUST be empty string in this case.
+    # That's why I didn't use get_user_token_by_code().
+    my $query_ref = +{
+        code         => $parsed_signed_request->{code},
+        redirect_uri => '',
+    };
+    my $token_ref = $self->_get_token($query_ref);
+
+    croak 'expires is not returned' unless $token_ref->{expires};
+
+    return $token_ref;
+}
 
 # Access Tokens > Expiration and Extending Tokens
 # https://developers.facebook.com/docs/facebook-login/access-tokens/
@@ -210,7 +238,6 @@ sub exchange_token {
     my ($self, $short_term_token) = @_;
 
     croak 'short term token is not given' unless $short_term_token;
-    croak 'app_id and secret must be set' unless $self->app_id && $self->secret;
 
     my $query_ref = +{
         grant_type        => 'fb_exchange_token',
@@ -224,6 +251,8 @@ sub exchange_token {
 
 sub _get_token {
     my ($self, $param_ref) = @_;
+    
+    croak 'app_id and secret must be set' unless $self->app_id && $self->secret;
 
     $param_ref = +{
         %$param_ref,
@@ -484,6 +513,20 @@ sub gen_appsecret_proof {
     return hmac_sha256_hex($self->access_token, $self->secret);
 }
 
+sub js_cookie_name {
+    my $self = shift;
+    croak 'app_id must be set' unless $self->app_id;
+
+    # Cookie is set by JS SDK with a anme of fbsr_{app_id}. Official document
+    # is not provided for more than 3 yaers so I quote from PHP SDK's code.
+    # "Constructs and returns the name of the cookie that potentially houses
+    # the signed request for the app user. The cookie is not set by the
+    # BaseFacebook class, but it may be set by the JavaScript SDK."
+    # The cookie value can be parsed as signed request and it contains 'code'
+    # to exchange for access toekn.
+    return sprintf('fbsr_%d', $self->app_id);
+}
+
 sub create_response {
     my $self = shift;
     return Facebook::OpenGraph::Response->new(+{
@@ -616,7 +659,7 @@ Facebook::OpenGraph - Simple way to handle Facebook's Graph API.
 
 =head1 VERSION
 
-This is Facebook::OpenGraph version 1.13
+This is Facebook::OpenGraph version 1.20
 
 =head1 SYNOPSIS
     
@@ -897,7 +940,22 @@ users to your Facebook page, App's Canvas page or any location on facebook.com.
 
 =head3 C<< $fb->parse_signed_request($signed_request_str) >>
 
-It parses signed_request that Facebook Platform gives to your callback endpoint.
+It parses signed_request that Facebook Platform gives to you on various 
+situations. situations may include
+
+=over 4
+
+=item * Given as a URL fragment on callback endpoint after login flow is done 
+with Login Dialog
+
+=item * POSTed when Page Tab App is loaded
+
+=item * Set in a form of cookie by JS SDK
+
+=back
+
+Fields in returning value are introduced at 
+L<https://developers.facebook.com/docs/reference/login/signed-request/>.
 
   my $req = Plack::Request->new($env);
   my $val = $fb->parse_signed_request($req->query_param('signed_request'));
@@ -956,6 +1014,36 @@ the user.
   my $access_token = $token_ref->{access_token};
   my $expires      = $token_ref->{expires};
 
+=head3 C<< $fb->get_user_token_by_cookie($cookie_value) >>
+
+Obtain user access token based on the cookie value that is set by JS SDK.
+Cookie name should be determined with C<js_cookie_name()>.
+
+  if (my $cookie = $c->req->cookie( $fb->js_cookie_name )) {
+    # User is not logged in yet, but cookie is set by JS SDK on previous visit.
+    my $token_ref = $fb->get_user_token_by_cookie($cookie);
+    # {
+    #     "access_token" : "new_token_string_qwerty",
+    #     "expires" : 5752
+    # };
+  }
+  else {
+    return $c->redirect( $fb->auth_uri );
+  }
+
+=head3 C<< $fb->exchange_token($short_term_token) >>
+
+Exchange short lived access token for long lived one. Short lived tokens are 
+ones that you obtain with C<get_user_token_by_code()>. Usually long lived 
+tokens live about 60 days while short lived ones live about 2 hours.
+
+  my $extended_token_ref = $fb->exchange_token($token_ref->{access_token});
+  my $access_token       = $extended_token_ref->{access_token};
+  my $expires            = $extended_token_ref->{expires};
+
+If you loved how old offline_access permission worked and are looking for a 
+substitute you might want to try this.
+
 =head3 C<< $fb->get($path, \%param, \@headers) >>
 
 Alias to C<request()> that sends C<GET> request.
@@ -1013,7 +1101,13 @@ Request batch request and returns an array reference.
 
 =head3 C<< $fb->batch(\@requests) >>
 
-Request batch request and returns an array reference.
+Request batch request and returns an array reference of response objects. It 
+sets C<$fb->access_token> as top level access token, but other than that you 
+can specify indivisual access token for each request. The document says 
+"The Batch API is flexible and allows individual requests to specify their own 
+access tokens as a query string or form post parameter. In that case the top 
+level access token is considered a fallback token and is used if an individual 
+request has not explicitly specified an access token."
 
   my $data = $fb->batch([
       +{method => 'GET', relative_url => 'zuck'},
@@ -1102,10 +1196,17 @@ object.
 
 =head3 C<< $fb->gen_appsecret_proof >>
 
-Generate signature for appsecret_proof parameter. This method is called in 
+Generates signature for appsecret_proof parameter. This method is called in 
 C<request()> if C<$self->use_appsecret_proof> is set. See 
 L<http://facebook-docs.oklahome.net/archives/52097348.html> for Japanese Info.
 
+=head3 C<< $fb->js_cookie_name >>
+
+Generates and returns the name of cookie that is set by JS SDK on client side 
+login. This value can be parsed as signed request and the parsed data structure 
+contains 'code' to exchange for acess token. See C<get_user_token_by_cookie()> 
+for detail.
+
 =head3 C<< $fb->create_response($http_status_code, $http_status_message, \@response_headers, $response_content) >>
 
 Creates and returns L<Facebook::OpenGraph::Response>. If you wish to use 
