@@ -20,12 +20,32 @@ sub new {
 }
 
 # accessors
-sub code        { shift->{code}        }
-sub message     { shift->{message}     }
-sub content     { shift->{content}     }
-sub req_headers { shift->{req_headers} }
-sub req_content { shift->{req_content} }
-sub json        { shift->{json}        }
+sub code        { shift->{code}             }
+sub headers     { shift->{headers}          }
+sub message     { shift->{message}          }
+sub content     { shift->{content}          }
+sub req_headers { shift->{req_headers}      }
+sub req_content { shift->{req_content}      }
+sub json        { shift->{json}             }
+sub etag        { shift->header('etag')     }
+
+sub header {
+    my ($self, $key) = @_;
+
+    croak 'header field name is not given' unless $key;
+
+    $self->{header} ||= do {
+        my $ref = +{};
+
+        while (my ($k, $v) = splice @{ $self->headers }, 0, 2) {
+            $ref->{$k} = $v;
+        }
+
+        $ref;
+    };
+
+    return $self->{header}->{$key};
+}
 
 sub is_success {
     my $self = shift;
@@ -34,10 +54,20 @@ sub is_success {
     return substr($self->code, 0, 1) == 2 || $self->code == 304;
 }
 
-# Errors: Error codes
-# https://developers.facebook.com/docs/reference/api/errors/
+# Using the Graph API > Handling Errors
+# https://developers.facebook.com/docs/graph-api/using-graph-api/
 sub error_string {
     my $self = shift;
+
+    # When error occurs, response should be given in a form of below:
+    #{
+    #  "error": {
+    #    "message": "Message describing the error", 
+    #    "type": "OAuthException", 
+    #    "code": 190 ,
+    #    "error_subcode": 460
+    #  }
+    #}
     my $error = eval { $self->as_hashref->{error}; };
 
     my $err_str = q{};
@@ -181,6 +211,17 @@ Returns request body. This is especially useful for debugging. You must install
 later version of Furl to enable this or otherwise empty string will be returned.
 Also you have to specify Furl::HTTP->new(capture_request => 1) option.
 
+=head3 C<< $res->etag >>
+
+Returns ETag value that is given as a part of response headers.
+
+=head3 C<< $res->header($field_name) >>
+
+Returns specified header field value.
+
+  my $res  = $fb->request('GET', 'go.hagiwara');
+  my $etag = $res->header('etag'); # "a376a57cb3a4bd3a3c6a53fca06b0fd5badee50b"
+
 =head3 C<< $res->is_success >>
 
 Returns if status is 2XX or 304. 304 is added to handle $fb->fetch_with_etag();
